@@ -1,5 +1,10 @@
 """
-Class EnvBase is the base class of the environment. This class will read the yaml file and create the world, robot, obstacle, and map objects.
+Class EnvBase is the base class of the environment.
+
+It loads YAML configuration via ``EnvConfig`` to construct the world,
+robots, obstacles, and maps, and provides the core simulation loop.
+The environment can be reconfigured in-place (same figure) by reloading
+the YAML at runtime.
 
 Author: Ruihua Han
 """
@@ -7,8 +12,7 @@ Author: Ruihua Han
 from __future__ import annotations
 
 import importlib
-import platform
-from operator import attrgetter
+from collections import Counter
 from typing import Any, Optional, Union
 
 import matplotlib
@@ -21,11 +25,11 @@ from irsim.config import env_param, world_param
 from irsim.env.env_config import EnvConfig
 from irsim.gui.mouse_control import MouseControl
 from irsim.lib import random_generate_polygon
-from irsim.world import ObjectBase, World
-from irsim.world.object_factory import ObjectFactory
+from irsim.util.random import rng, set_seed
+from irsim.util.util import normalize_actions, to_numpy
+from irsim.world import ObjectBase, ObjectFactory
 
 from .env_logger import EnvLogger
-from .env_plot import EnvPlot
 
 try:
     from irsim.gui.keyboard_control import KeyboardControl
@@ -68,8 +72,10 @@ class EnvBase:
     The base class for simulation environments in IR-SIM.
 
     This class serves as the foundation for creating and managing robotic simulation
-    environments. It reads YAML configuration files to create worlds, robots, obstacles,
-    and map objects, and provides the core simulation loop functionality.
+    environments. It reads YAML configuration files (through ``EnvConfig``) to create
+    worlds, robots, obstacles, and map objects, and provides the core simulation loop
+    functionality. The environment supports in-place reload of YAML configuration to
+    update the scene in the existing figure without opening a new window.
 
     Args:
         world_name (str, optional): Path to the world YAML configuration file.
@@ -88,16 +94,27 @@ class EnvBase:
             If None, logs will only be output to console.
         log_level (str): Logging level for the environment. Options include
             'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'. Default is 'INFO'.
+        seed (int, optional): Seed for the random number generator. Default is None.
+            If None, the seed will be set to a random value, which will make the simulation non-reproducible.
+            If a fixed seed is provided, the random simulation scenario will be reproducible.
 
     Attributes:
-        display (bool): Display flag for the environment.
-        disable_all_plot (bool): Plot disable flag.
-        save_ani (bool): Animation saving flag.
-        objects (list): List of all objects in the environment.
-        world (World): The world object containing environment configuration.
-        robot_collection (list): List of robot objects.
-        obstacle_collection (list): List of obstacle objects.
-        map_collection (list): List of map objects.
+        display (bool): Whether to display the environment visualization.
+        disable_all_plot (bool): Whether all plotting is disabled.
+        save_ani (bool): Whether to save animation during simulation.
+
+        env_config (EnvConfig): Configuration loader managing YAML parsing and object creation.
+
+        keyboard (KeyboardControl): Keyboard input handler for manual control.
+        mouse (MouseControl): Mouse input handler for zoom and pan.
+
+        pause_flag (bool): Internal flag indicating if simulation is paused.
+        quit_flag (bool): Internal flag indicating if simulation should quit.
+        debug_flag (bool): Internal flag for debug mode (frame-by-frame stepping).
+        debug_count (int): Counter for debug mode frames.
+        reset_flag (bool): Internal flag for environment reset.
+        reload_flag (bool): Internal flag for YAML reload.
+        save_figure_flag (bool): Internal flag to save current figure.
 
     Example:
         >>> # Create a basic environment
@@ -108,6 +125,9 @@ class EnvBase:
         >>>
         >>> # Create environment with animation saving
         >>> env = EnvBase("world.yaml", save_ani=True, full=True)
+        >>>
+        >>> # Create environment with a fixed seed for reproducibility
+        >>> env = EnvBase("world.yaml", seed=42)
     """
 
     def __init__(
@@ -119,79 +139,89 @@ class EnvBase:
         full: bool = False,
         log_file: Optional[str] = None,
         log_level: str = "INFO",
+        seed: Optional[int] = None,
     ) -> None:
         # init env setting
         self.display = display
+        set_seed(seed)
 
         if not self.display:
             matplotlib.use("Agg")
 
         self.disable_all_plot = disable_all_plot
         self.save_ani = save_ani
+
         env_param.logger = EnvLogger(log_file, log_level)
 
-        self.env_config = EnvConfig(world_name)
-        self.object_factory = ObjectFactory()
-        # init objects (world, obstacle, robot)
+        try:
+            self.env_config = EnvConfig(world_name)
+        except Exception as e:
+            self.logger.critical(f"YAML Configuration load failed: {e}")
+            raise
 
-        self._world = World(world_name, **self.env_config.parse["world"])
+        (
+            self._world,
+            self._objects,
+            self._env_plot,
+            self._robot_collection,
+            self._obstacle_collection,
+            self._map_collection,
+        ) = self.env_config.initialize_objects()
 
-        self._robot_collection = self.object_factory.create_from_parse(
-            self.env_config.parse["robot"], "robot"
-        )
-        self._obstacle_collection = self.object_factory.create_from_parse(
-            self.env_config.parse["obstacle"], "obstacle"
-        )
-        self._map_collection = self.object_factory.create_from_map(
-            self._world.obstacle_positions, self._world.buffer_reso
-        )
-
-        self._objects = (
-            self._robot_collection + self._obstacle_collection + self._map_collection
-        )
-
-        self._objects.sort(key=attrgetter("id"))
         self.build_tree()
+        env_param.objects = self._objects
+        self.validate_unique_names()
 
-        # env parameters
-        self._env_plot = EnvPlot(self._world, self.objects, **self._world.plot_parse)
+        # Try to initialize keyboard control (pynput or MPL backend inside KeyboardControl)
+        try:
+            keyboard_config = self.env_config.parse["gui"].get("keyboard", {})
+            self.keyboard = KeyboardControl(env_ref=self, **keyboard_config)
+        except Exception as e:
+            self.logger.error(
+                f"Keyboard control unavailable error: {e}. Auto control applied. "
+                "Install 'pynput' or set backend='mpl' in YAML keyboard config."
+            )
+            world_param.control_mode = "auto"
 
-        env_param.objects = self.objects
+        mouse_config = self.env_config.parse["gui"].get("mouse", {})
+        self.mouse = MouseControl(self._env_plot.ax, **mouse_config)
 
-        if world_param.control_mode == "keyboard":
-            if not keyboard_module:
-                self.logger.error(
-                    "Keyboard module is not installed. Auto control applied. Please install the dependency by 'pip install ir-sim[keyboard]'."
-                )
-                world_param.control_mode = "auto"
-            else:
-                self.keyboard = KeyboardControl(
-                    env_ref=self, **self.env_config.parse["keyboard"]
-                )
-
-        self.mouse = MouseControl(self._env_plot.ax)
-
-        # flag
+        # flag for keyboard control
         self.pause_flag = False
+        self.quit_flag = False
+        self.debug_flag = False
+        self.debug_count = 0
+        self.reset_flag = False
+        self.reload_flag = False
+        self.save_figure_flag = False
 
         if full:
-            system_platform = platform.system()
-            if system_platform == "Linux" or system_platform == "Windows":
-                mng = plt.get_current_fig_manager()
-                if mng is not None:
-                    mng.full_screen_toggle()
+            mng = plt.get_current_fig_manager()
+            if mng is not None:
+                mng.full_screen_toggle()
 
         # Log simulation start
         self.logger.info(
-            f"Simulation environment '{self._world.name}' has been initialized and started."
+            f"Simulation environment '{self._world.name}' started. Step time {self._world.step_time:.3f} s."
         )
 
     def __del__(self):
+        """Clean up resources when the environment is garbage-collected.
+
+        Note:
+            Main resource cleanup is performed in :py:meth:`end`.
+        """
         pass
 
     def __str__(self):
+        """Return a human-readable summary of the environment.
+
+        Returns:
+            str: Summary string including the world name.
+        """
         return f"Environment: {self._world.name}"
 
+    @normalize_actions
     def step(
         self,
         action: Optional[Union[np.ndarray, list[Any]]] = None,
@@ -213,6 +243,11 @@ class EnvBase:
 
                 If None, robots will use their default behavior or keyboard control if enabled.
 
+        Note - Priority Order:
+                    1. Apply keyboard control for the specified ``action_id`` if enabled.
+                    2. Apply the provided ``action`` (list of numpy arrays) to robots by ``action_id`` (int or list of int).
+                    3. For remaining robots, fall back to their configured behaviors when ``action`` is ``None``.
+
             action_id (Union[int, list], optional): ID(s) of the robot(s) to apply the action(s) to.
                 Can be a single robot ID or a list of IDs. Default is 0 (first robot).
                 If action is a list and action_id is a single int, all actions will be
@@ -235,36 +270,55 @@ class EnvBase:
             >>> env.step(actions, action_id=[0, 1])  # Move robots 0 and 1
         """
 
-        if self.pause_flag:
+        if self.quit_flag:
+            self.quit()
+
+        if (
+            self.debug_flag and world_param.count > self.debug_count
+        ) or self.pause_flag:
             return
 
-        if isinstance(action, list):
-            if isinstance(action_id, list):
-                for a, ai in zip(action, action_id):
-                    self._object_step(a, ai)
-            else:
-                self._objects_step(action)
-        else:
-            if world_param.control_mode == "keyboard":
-                self._object_step(self.key_vel, self.key_id)
-            else:
-                if isinstance(action_id, list):
-                    self._object_step(action, action_id[0])
-                else:
-                    self._object_step(action, action_id)
+        actions = action  # normalized by decorator to a list aligned with self.objects
+
+        if world_param.control_mode == "keyboard" and self.key_id < len(actions):
+            actions[self.key_id] = self.key_vel
+
+        self._objects_step(actions, sensor_step=False)
+        self._objects_sensor_step()
+        self._world.step()
+        self._status_step()
+
+    def _objects_step(self, action: list[Any], sensor_step: bool = True) -> None:
+        """Advance all objects by one step with corresponding actions.
+
+        Args:
+            action (list[Any]): A list of actions aligned with ``self.objects``.
+                If the list is shorter than the number of objects, it is padded
+                with ``None`` for the remaining objects.
+        """
+        action = action + [None] * (len(self.objects) - len(action))
+        [obj.step(action, sensor_step) for obj, action in zip(self.objects, action)]
 
         self.build_tree()
-        self._objects_check_status()
-        self._world.step()
-        self.step_status()
 
-    def _objects_step(self, action: list[Any]) -> None:
-        action = action + [None] * (len(self.objects) - len(action))
-        [obj.step(action) for obj, action in zip(self.objects, action)]
+    def _objects_sensor_step(self) -> None:
+        """step the sensors of all objects with updated states"""
+        [obj.sensor_step() for obj in self.objects]
 
     def _object_step(
         self, action: np.ndarray | list[Any] | None, obj_id: int = 0
     ) -> None:
+        """Advance a single object by one step and tick others.
+
+        Deprecated:
+            This method is slated for removal in a future release. Prefer
+            :py:meth:`step` (with ``action`` and ``action_id``) to control
+            specific robots.
+
+        Args:
+            action (np.ndarray | list | None): Action applied to the target object.
+            obj_id (int): Target object index (or id-aligned index). Default is ``0``.
+        """
         if len(self.objects) == 0:
             return
 
@@ -272,12 +326,13 @@ class EnvBase:
         [obj.step() for obj in self.objects if obj._id != obj_id]
 
     def _objects_check_status(self) -> None:
+        """Refresh per-object status flags (e.g., arrival, collision)."""
         [obj.check_status() for obj in self.objects]
 
     # render
     def render(
         self,
-        interval: float = 0.02,
+        interval: float = 0.01,
         figure_kwargs: Optional[dict[str, Any]] = None,
         mode: str = "dynamic",
         **kwargs: Any,
@@ -286,10 +341,13 @@ class EnvBase:
         Render the environment.
 
         Args:
-            interval(float) :  Time interval between frames in seconds.
-            figure_kwargs(dict) : Additional keyword arguments for saving figures, see `savefig <https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.savefig.html>`_ for detail.
-            mode(str) : "dynamic", "static", "all" to specify which type of objects to draw and clear.
-            kwargs: Additional keyword arguments for drawing components. see :py:meth:`.ObjectBase.plot` function for detail.
+            interval (float): Time interval between frames in seconds.
+            figure_kwargs (dict): Additional keyword arguments for saving figures,
+                see `savefig <https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.savefig.html>`_ for details.
+            mode (str): One of {"dynamic", "static", "all"} specifying which types of objects
+                to draw and clear each frame.
+            kwargs: Additional keyword arguments for drawing components. See
+                :py:meth:`.ObjectBase.plot` for details.
         """
 
         if figure_kwargs is None:
@@ -303,6 +361,16 @@ class EnvBase:
 
             self._env_plot.step(mode, self.objects, **kwargs)
 
+        if self.save_figure_flag:
+            self.save_figure(save_gif=True, **figure_kwargs)
+            self.save_figure_flag = False
+
+        if self.reset_flag:
+            self.reset()
+
+        if self.reload_flag:
+            self.reload()
+
     def show(self) -> None:
         """
         Show the environment figure.
@@ -318,9 +386,11 @@ class EnvBase:
         Draw the trajectory on the environment figure.
 
         Args:
-            traj (list): List of trajectory points (2 * 1 vector).
-            traj_type: Type of the trajectory line, see matplotlib plot function for detail.
-            **kwargs: Additional keyword arguments for drawing the trajectory, see :py:meth:`.EnvPlot.draw_trajectory` for detail.
+            traj (list): List of trajectory points. Each point is a 2x1 vector
+                or an array of shape (2, N).
+            traj_type (str): Matplotlib line style (e.g., "g-", "r--").
+            **kwargs: Additional keyword arguments; forwarded to
+                :py:meth:`.EnvPlot.draw_trajectory`.
         """
 
         self._env_plot.draw_trajectory(traj, traj_type, **kwargs)
@@ -337,12 +407,13 @@ class EnvBase:
         Draw points on the environment figure.
 
         Args:
-            points (list): List of points (2*1) to be drawn.
-                or (np.array): (2, Num) to be drawn.
-            s (int): Size of the points.
-            c (str): Color of the points.
-            refresh (bool): Flag to refresh the points in the figure.
-            **kwargs: Additional keyword arguments for drawing the points, see `ax.scatter <https://matplotlib.org/stable/api/_as_gen/matplotlib.axes.Axes.scatter.html>`_ function for detail.
+            points (list | np.ndarray): Either a list of 2x1 points or a numpy
+                array with shape (2, N).
+            s (int): Marker size.
+            c (str): Marker color.
+            refresh (bool): Whether to clear previous points before drawing.
+            **kwargs: Additional keyword arguments, forwarded to
+                `Axes.scatter <https://matplotlib.org/stable/api/_as_gen/matplotlib.axes.Axes.scatter.html>`_.
         """
 
         self._env_plot.draw_points(points, s, c, refresh, **kwargs)
@@ -354,9 +425,9 @@ class EnvBase:
         Draw a box by the vertices.
 
         Args:
-            vertex (np.ndarray): matrix of vertices, point_dim*vertex_num
-            refresh (bool): whether to refresh the plot, default True
-            color (str): color of the box, default '-b'
+            vertex (np.ndarray): Vertices matrix with shape (point_dim, num_vertices).
+            refresh (bool): Whether to clear previous boxes before drawing. Default is False.
+            color (str): Line style/color for the box (e.g., "-b").
         """
         self._env_plot.draw_box(vertex, refresh, color)
 
@@ -365,9 +436,10 @@ class EnvBase:
         Draw a single quiver (arrow) on the environment figure.
 
         Args:
-            point: Point data for the quiver
-            refresh (bool): Flag to refresh the quiver in the figure, default False
-            **kwargs: Additional keyword arguments for drawing the quiver
+            point: A tuple ``(x, y, u, v)`` or compatible structure defining the
+                arrow's origin and vector.
+            refresh (bool): Whether to clear previous quiver before drawing. Default False.
+            **kwargs: Additional keyword arguments for drawing the quiver.
         """
         self._env_plot.draw_quiver(point, refresh, **kwargs)
 
@@ -376,9 +448,10 @@ class EnvBase:
         Draw multiple quivers (arrows) on the environment figure.
 
         Args:
-            points: Points data for the quivers
-            refresh (bool): Flag to refresh the quivers in the figure, default False
-            **kwargs: Additional keyword arguments for drawing the quivers
+            points: Iterable of tuples/lists/arrays compatible with
+                ``(x, y, u, v)`` per arrow.
+            refresh (bool): Whether to clear previous quivers before drawing. Default False.
+            **kwargs: Additional keyword arguments for drawing the quivers.
         """
         self._env_plot.draw_quivers(points, refresh, **kwargs)
 
@@ -399,19 +472,44 @@ class EnvBase:
 
         if self.display:
             plt.pause(ending_time)
-            self.logger.info(f"Figure will be closed within {ending_time:.2f} seconds.")
+            self.logger.info(
+                f"Simulation Environment '{self._world.name}' closing in {ending_time:.2f} seconds."
+            )
 
         plt.close("all")
         env_param.objects = []
         ObjectBase.reset_id_iter()
 
-        if world_param.control_mode == "keyboard":
-            self.keyboard.listener.stop()
+        if hasattr(self, "keyboard"):
+            # Stop pynput listener if present; otherwise disconnect MPL callbacks
+            try:
+                if (
+                    hasattr(self.keyboard, "listener")
+                    and self.keyboard.listener is not None
+                ):
+                    self.keyboard.listener.stop()
+                else:
+                    fig = plt.gcf()
+                    if hasattr(self.keyboard, "_mpl_press_cid"):
+                        fig.canvas.mpl_disconnect(self.keyboard._mpl_press_cid)
+                    if hasattr(self.keyboard, "_mpl_release_cid"):
+                        fig.canvas.mpl_disconnect(self.keyboard._mpl_release_cid)
+            except Exception:
+                pass
 
         self.logger.info(
-            f"The simulated environment has ended. Total simulation time: {round(self._world.time, 2)} seconds."
+            f"Simulation Environment '{self._world.name}' ended. Total time {self._world.time:.2f} seconds."
         )
 
+    def quit(self) -> None:
+        """
+        Quit the environment.
+        """
+        self.quit_flag = True
+        self.logger.info("Quit the environment.")
+        self.end(ending_time=1.0)
+        raise SystemExit(0)
+
     def done(self, mode: str = "all") -> Optional[bool]:
         """
         Check if the simulation should terminate based on robot completion status.
@@ -452,7 +550,7 @@ class EnvBase:
             return any(done_list)
         return None
 
-    def step_status(self) -> None:
+    def _status_step(self) -> None:
         """
         Update and log the current status of all robots in the environment.
 
@@ -465,6 +563,9 @@ class EnvBase:
             The status information is automatically updated during simulation steps.
         """
 
+        # object status step
+        [obj.check_status() for obj in self.objects]
+
         arrive_list = [obj.arrive for obj in self.objects if obj.role == "robot"]
         collision_list = [obj.collision for obj in self.objects if obj.role == "robot"]
 
@@ -474,11 +575,24 @@ class EnvBase:
             collision_list = [False]
 
         if all(arrive_list):
-            self._world.status = "Arrived"
+            self.set_status("Arrived")
         elif any(collision_list):
-            self._world.status = "Collision"
+            self.set_status("Collision")
+        elif self.reset_flag:
+            self.set_status("Reset")
+        elif self.reload_flag:
+            self.set_status("Reload")
+        elif self.save_figure_flag:
+            self.set_status("Save Figure")
+        elif self.quit_flag:
+            self.set_status("Quit")
+        elif self.debug_flag:
+            self.set_status("Pause (Debugging)")
         else:
-            self._world.status = "Running"
+            if world_param.control_mode == "keyboard":
+                self.set_status("Running (keyboard)")
+            else:
+                self.set_status("Running")
 
     def pause(self) -> None:
         """
@@ -491,7 +605,7 @@ class EnvBase:
             >>> env.pause()
             >>> env.step([1.0, 0.0])  # This will have no effect while paused
         """
-        self._world.status = "Pause"
+        self.set_status("Pause")
         self.pause_flag = True
 
     def resume(self) -> None:
@@ -507,8 +621,13 @@ class EnvBase:
             >>> env.resume()
             >>> env.step([1.0, 0.0])  # This will now work again
         """
-        self._world.status = "Running"
-        self.pause_flag = False
+        self.set_status("Running")
+        if self.pause_flag:
+            self.pause_flag = False
+
+        if self.debug_flag:
+            self.debug_flag = False
+            self.debug_count = 0
 
     def reset(self) -> None:
         """
@@ -534,18 +653,25 @@ class EnvBase:
         self.step(action=np.zeros((2, 1)))
         self._world.reset()
         self.reset_plot()
-        self._world.status = "Reset"
+        self.set_status("Reset")
+        self.pause_flag = False
+        self.debug_flag = False
+        self.debug_count = 0
+        self.reset_flag = False
 
     def _reset_all(self) -> None:
         [obj.reset() for obj in self.objects]
 
     def reset_plot(self) -> None:
         """
-        Reset the environment figure.
+        Reset the environment figure in-place.
+
+        Re-initializes drawing on the current figure/axes using the existing
+        ``EnvPlot`` instance; does not create a new figure window.
         """
 
         self._env_plot.clear_components("all", self.objects)
-        self._env_plot.init_plot(self._world.grid_map, self.objects)
+        self._env_plot._init_plot(self._world, self.objects)
 
     # region: environment change
     def random_obstacle_position(
@@ -564,33 +690,24 @@ class EnvBase:
             ids (list): A list of IDs of objects for which to set random positions. Default is None.
             non_overlapping (bool): If set, the obstacles that will be reset to random obstacles will not overlap with other obstacles. Default is False.
         """
-        if range_high is None:
-            range_high = [10, 10, 3.14]
-        if range_low is None:
-            range_low = [0, 0, -3.14]
+
+        range_low = to_numpy(range_low, np.array([0, 0, -3.14]), (3, 1))
+        range_high = to_numpy(range_high, np.array([10, 10, 3.14]), (3, 1))
+
         if ids is None:
             ids = [obs.id for obs in self.obstacle_list]
-        if isinstance(range_low, list):
-            range_low = np.c_[range_low]
-
-        if isinstance(range_high, list):
-            range_high = np.c_[range_high]
 
         selected_obs = [obs for obs in self.obstacle_list if obs.id in ids]
         existing_obj = [obj for obj in self.objects if obj.id not in ids]
 
         for obj in selected_obs:
             if not non_overlapping:
-                obj.set_state(
-                    np.random.uniform(range_low, range_high, (3, 1)), init=True
-                )
+                obj.set_state(rng.uniform(range_low, range_high, (3, 1)), init=True)
             else:
                 counter = 0
 
                 while counter < 100:
-                    obj.set_state(
-                        np.random.uniform(range_low, range_high, (3, 1)), init=True
-                    )
+                    obj.set_state(rng.uniform(range_low, range_high, (3, 1)), init=True)
 
                     if any(obj.check_collision(exi_obj) for exi_obj in existing_obj):
                         counter += 1
@@ -598,8 +715,7 @@ class EnvBase:
                         existing_obj.append(obj)
                         break
 
-        self._env_plot.clear_components("all", self.obstacle_list)
-        self._env_plot.draw_components("all", self.obstacle_list)
+        self._env_plot.step("all", self.obstacle_list)
 
     def random_polygon_shape(
         self,
@@ -660,8 +776,34 @@ class EnvBase:
                 geom = Polygon(vertices_list[i])
                 obj.set_original_geometry(geom)
 
-        self._env_plot.clear_components("all", self.obstacle_list)
-        self._env_plot.draw_components("all", self.obstacle_list)
+        self._env_plot.step("all", self.obstacle_list)
+
+    def reload(self, world_name: Optional[str] = None) -> None:
+        """
+        Reload the environment from YAML and update the current figure.
+
+        This re-parses the YAML and re-creates world/objects, then refreshes
+        drawing on the existing figure/axes (no new window is created).
+
+        Args:
+            world_name (str): Optional name/path of the world YAML to reload.
+                If ``None``, the previous YAML file is used.
+        """
+        ObjectBase.reset_id_iter()
+        self.reset()
+        self._env_plot.clear_components("all", self.objects)
+        (
+            self._world,
+            self._objects,
+            self._env_plot,
+            self._robot_collection,
+            self._obstacle_collection,
+            self._map_collection,
+        ) = self.env_config.reload_yaml_objects(world_name)
+        self.build_tree()
+        self.validate_unique_names()
+        env_param.objects = self._objects
+        self.reload_flag = False
 
     # endregion: environment change
 
@@ -683,21 +825,32 @@ class EnvBase:
 
     def add_object(self, obj: ObjectBase) -> None:
         """
-        Add the object to the environment.
+        Add the object to the environment, enforcing unique names.
 
         Args:
             obj (ObjectBase): The object to be added to the environment.
         """
+        if any(existing.name == obj.name for existing in self.objects):
+            raise ValueError(f"Object name '{obj.name}' already exists.")
         self._objects.append(obj)
         self.build_tree()
 
     def add_objects(self, objs: list[ObjectBase]) -> None:
         """
-        Add the objects to the environment.
+        Add the objects to the environment, enforcing unique names (both within
+        the new list and against existing objects).
 
         Args:
             objs (list): List of objects to be added to the environment.
         """
+        new_names = [o.name for o in objs]
+        dupes_in_new = [n for n, c in Counter(new_names).items() if c > 1]
+        if dupes_in_new:
+            raise ValueError(f"Duplicate names within new objects: {dupes_in_new}")
+        existing_names = {o.name for o in self.objects}
+        conflicts = [n for n in new_names if n in existing_names]
+        if conflicts:
+            raise ValueError(f"Object names already exist: {conflicts}")
         self._objects.extend(objs)
         self.build_tree()
 
@@ -740,6 +893,17 @@ class EnvBase:
 
         env_param.GeometryTree = STRtree([obj.geometry for obj in self.objects])
 
+    def validate_unique_names(self) -> None:
+        """Validate that all object names are unique.
+
+        Raises:
+            ValueError: If duplicates exist.
+        """
+        names = [obj.name for obj in self.objects]
+        duplicates = [n for n, c in Counter(names).items() if c > 1]
+        if duplicates:
+            raise ValueError(f"Duplicate object names: {duplicates}")
+
     # endregion: object operation
 
     # region: get information
@@ -757,7 +921,7 @@ class EnvBase:
 
     def get_lidar_scan(self, id: int = 0) -> dict[str, Any]:
         """
-        Get the lidar scan of the robot with the given id.
+        Get the LiDAR scan of the robot with the given id.
 
         Args:
             id (int): Id of the robot.
@@ -770,7 +934,7 @@ class EnvBase:
 
     def get_lidar_offset(self, id: int = 0) -> list[float]:
         """
-        Get the lidar offset of the robot with the given id.
+        Get the LiDAR offset of the robot with the given id.
 
 
         Args:
@@ -827,6 +991,18 @@ class EnvBase:
         """
         return self._world.get_map(resolution, self.obstacle_list)
 
+    def get_object_by_name(self, name: str) -> Optional[ObjectBase]:
+        """
+        Get the object with the given name.
+        """
+        return next((obj for obj in self.objects if obj.name == name), None)
+
+    def get_object_by_id(self, target_id: int) -> Optional[ObjectBase]:
+        """
+        Get the object with the given id.
+        """
+        return next((obj for obj in self.objects if obj.id == target_id), None)
+
     # endregion: get information
 
     def set_title(self, title: str) -> None:
@@ -836,6 +1012,25 @@ class EnvBase:
 
         self._env_plot.title = title
 
+    def set_random_seed(self, seed: Optional[int] = None) -> None:
+        """
+        Set IR-SIM's random seed for reproducibility.
+
+        Args:
+            seed (int, optional): Seed for IR-SIM's project RNG. If ``None``, a
+                new unseeded generator is created (non-reproducible). This
+                controls randomness that goes through IR-SIM's RNG. Custom code
+                using ``np.random.*`` or Python ``random`` must be seeded separately
+                or migrated to use IR-SIM's RNG.
+        """
+        set_seed(seed)
+
+    def set_status(self, status: str) -> None:
+        """
+        Set the status of the environment.
+        """
+        self._world.status = status
+
     def save_figure(
         self,
         save_name: Optional[str] = None,
@@ -992,22 +1187,57 @@ class EnvBase:
 
     @property
     def key_vel(self) -> Any:
+        """Get current keyboard velocity command.
+
+        Returns:
+            Any: A 2x1 vector ``[[linear], [angular]]`` from keyboard input.
+        """
         return self.keyboard.key_vel
 
     @property
     def key_id(self) -> int:
+        """Get current keyboard-controlled robot id.
+
+        Returns:
+            int: The robot id currently controlled by keyboard.
+        """
         return self.keyboard.key_id
 
     @property
     def mouse_pos(self) -> Any:
+        """Get current mouse position on the canvas.
+
+        Returns:
+            Any: Mouse coordinates ``(x, y)`` or ``None`` if outside axes.
+        """
         return self.mouse.mouse_pos
 
     @property
     def mouse_left_pos(self) -> Any:
+        """Get last left-click position.
+
+        Returns:
+            Any: Position array or ``None`` if not set.
+        """
         return self.mouse.left_click_pos
 
     @property
     def mouse_right_pos(self) -> Any:
+        """Get last right-click position.
+
+        Returns:
+            Any: Position array or ``None`` if not set.
+        """
         return self.mouse.right_click_pos
 
+    @property
+    def names(self) -> list[str]:
+        """Get the names of all objects in the environment."""
+        return [obj.name for obj in self.objects]
+
+    @property
+    def object_factory(self) -> ObjectFactory:
+        """Get the object factory of the environment."""
+        return self.env_config.object_factory
+
     # endregion: property
