@@ -9,17 +9,22 @@ from __future__ import annotations
 import glob
 import os
 import shutil
-from collections.abc import Iterable, Sequence
+from collections.abc import Iterable
 from math import cos, sin
 from typing import Any, Optional
 
 import imageio.v3 as imageio
 import matplotlib.pyplot as plt
+import matplotlib.transforms as mtransforms
+import mpl_toolkits.mplot3d.art3d as art3d
 import numpy as np
+from matplotlib.lines import Line2D
+from matplotlib.patches import Arrow, Circle, Ellipse, Polygon, Rectangle, Wedge
 from mpl_toolkits.mplot3d import Axes3D
 
 from irsim.config import env_param, world_param
 from irsim.config.path_param import path_manager as pm
+from irsim.util.util import points_to_xy_list, traj_to_xy_list
 
 
 class EnvPlot:
@@ -27,50 +32,42 @@ class EnvPlot:
     EnvPlot class for visualizing the environment.
 
     Args:
-        world: The world object containing environment information including grid_map, x_range, y_range.
-        objects (list, optional): List of objects in the environment. Default is [].
-        saved_figure (dict, optional): Keyword arguments for saving the figure.
-            See https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.savefig.html for details.
-            Default is dict().
-        figure_pixels (list, optional): Width and height of the figure in pixels. Default is [1180, 1080].
-        show_title (bool, optional): Whether to show the title. Default is True.
-        kwargs: Additional options such as color_map, no_axis, and tight.
+        world: The world object with ranges and a ``plot_parse`` dictionary that
+            configures plotting (e.g., saved_figure, figure_pixels, show_title,
+            no_axis, tight).
+        objects (list, optional): Initial objects to draw. Default is ``[]``.
+        kwargs: Plot overrides that update ``world.plot_parse`` at runtime
+            (e.g., ``saved_figure``, ``figure_pixels``, ``show_title``,
+            ``no_axis``, ``tight``).
     """
 
     def __init__(
         self,
         world: Any,
         objects: Optional[list[Any]] = None,
-        saved_figure: Optional[dict[str, Any]] = None,
-        figure_pixels: Optional[Sequence[int]] = None,
-        show_title: bool = True,
         **kwargs: Any,
     ) -> None:
         """
         Initialize the EnvPlot instance.
 
         Sets up the matplotlib figure, configures plotting parameters,
-        and initializes the plot with world data and objects.
+        and initializes the plot with world data and objects. Any provided
+        ``kwargs`` override values in ``world.plot_parse``.
         """
-        # Store world and basic properties
-        if saved_figure is None:
-            saved_figure = {}
-        if figure_pixels is None:
-            figure_pixels = [1000, 800]
-        if objects is None:
-            objects = []
-        self.world = world
-        self.x_range = world.x_range
-        self.y_range = world.y_range
-        self.show_title = show_title
-        self.title = None
 
-        # Configure figure saving options
+        world.plot_parse.update(kwargs)
+
+        # default saved figure kwargs
         self.saved_figure_kwargs: dict[str, Any] = {
             "dpi": 100,
             "bbox_inches": "tight",
         }
-        self.saved_figure_kwargs.update(saved_figure)
+
+        self.saved_figure_kwargs.update(world.plot_parse.get("saved_figure", {}))
+        figure_pixels = world.plot_parse.get("figure_pixels", [1000, 800])
+
+        if objects is None:
+            objects = []
 
         # Create matplotlib figure and axes
         self.fig, self.ax = plt.subplots(
@@ -81,67 +78,77 @@ class EnvPlot:
             dpi=self.saved_figure_kwargs["dpi"],
         )
 
-        # Initialize plot settings and appearance
-        self.color_map: dict[str, str] = {
-            "robot": "g",
-            "obstacle": "k",
-            "landmark": "b",
-            "target": "pink",
-        }
-        self.color_map.update(kwargs.get("color_map", {}))
-
-        # Configure save options
-        self.saved_ani_kwargs: dict[str, Any] = {}
-
         # Initialize dynamic plotting lists
         self.dyna_line_list: list[Any] = []
         self.dyna_point_list: list[Any] = []
         self.dyna_quiver_list: list[Any] = []
 
         # Initialize the plot with world data
-        self.init_plot(world.grid_map, objects, **kwargs)
+        self._init_plot(world, objects, **kwargs)
 
-    def init_plot(
+        # Initialize plot settings and appearance
+        # self.color_map: dict[str, str] = {
+        #     "robot": "g",
+        #     "obstacle": "k",
+        #     "landmark": "b",
+        #     "target": "pink",
+        # }
+        # self.color_map.update(kwargs.get("color_map", {}))
+
+    def _init_plot(
         self,
-        grid_map: Optional[Any],
+        world: Any,
         objects: list[Any],
-        no_axis: bool = False,
-        tight: bool = True,
         **kwargs: Any,
     ) -> None:
         """
-        Initialize the plot with the given grid map and objects.
+        Initialize or re-initialize drawing on the existing figure/axes.
+
+        Safe to call when the world and objects change (e.g., YAML reload);
+        it reuses the current figure without creating a new window.
 
         Args:
-            grid_map (optional): The grid map of the environment.
-            objects (list): List of objects to plot.
-            no_axis (bool, optional): Whether to show the axis. Default is False.
-            tight (bool, optional): Whether to show the axis tightly. Default is True.
+            world: World instance providing ranges and grid map.
+            objects (list): List of objects to initialize and draw.
+            **kwargs: Plot overrides merged into ``world.plot_parse``.
         """
 
+        world.plot_parse.update(kwargs)
+        self.world = world
+        self.saved_ani_kwargs: dict[str, Any] = {}
+        self.title = world.plot_parse.get("title", None)
+        self.show_title = world.plot_parse.get("show_title", True)
+        self.saved_figure_kwargs.update(world.plot_parse.get("saved_figure", {}))
+
         if isinstance(self.ax, Axes3D):
             self.ax.set_box_aspect([1, 1, 1])
         else:
             self.ax.set_aspect("equal")
 
-        self.ax.set_xlim(self.x_range)
-        self.ax.set_ylim(self.y_range)
+        self.ax.set_xlim(world.x_range)
+        self.ax.set_ylim(world.y_range)
 
         self.ax.set_xlabel("x [m]")
         self.ax.set_ylabel("y [m]")
 
-        # self.draw_components("all", objects)
         self.init_objects_plot(objects)
-        self.draw_grid_map(grid_map)
+        self.draw_grid_map(world.grid_map)
 
-        if no_axis:
+        if world.plot_parse.get("no_axis", False):
             plt.axis("off")
-        if tight:
+        if world.plot_parse.get("tight", True):
             self.fig.tight_layout()
 
     def step(
         self, mode: str = "dynamic", objects: Optional[list[Any]] = None, **kwargs: Any
     ) -> None:
+        """Advance the plot by one step for the given objects.
+
+        Args:
+            mode (str): Which objects to update: "dynamic", "static", or "all".
+            objects (list | None): The objects to update/draw. Defaults to empty list.
+            **kwargs: Extra drawing options passed through to objects' plot methods.
+        """
         if objects is None:
             objects = []
         if self.show_title:
@@ -155,6 +162,12 @@ class EnvPlot:
             self.step_objects_plot(mode, objects, **kwargs)
 
     def init_objects_plot(self, objects: list[Any], **kwargs: Any) -> None:
+        """Initialize plot state for provided objects, then render once.
+
+        Args:
+            objects (list): Objects to be initialized on the axes.
+            **kwargs: Extra drawing options passed to initialization/plot.
+        """
         if self.show_title:
             self.update_title()
 
@@ -165,7 +178,7 @@ class EnvPlot:
         self, mode: str = "dynamic", objects: Optional[list[Any]] = None, **kwargs: Any
     ) -> None:
         """
-        Update the plot for the objects.
+        Update the plot for the objects by transform based on the object's original geometry.
         """
         if objects is None:
             objects = []
@@ -182,7 +195,7 @@ class EnvPlot:
         self, mode: str = "all", objects: Optional[list[Any]] = None, **kwargs: Any
     ) -> None:
         """
-        Draw the components in the environment.
+        Draw the components in the environment with global axis.
 
         Args:
             mode (str): 'static', 'dynamic', or 'all' to specify which objects to draw.
@@ -276,12 +289,7 @@ class EnvPlot:
             refresh (bool): Whether to refresh the plot.
             kwargs: Additional plotting options for ax.plot()
         """
-        if isinstance(traj, list):
-            path_x_list = [p[0, 0] for p in traj]
-            path_y_list = [p[1, 0] for p in traj]
-        elif isinstance(traj, np.ndarray):
-            path_x_list = [p[0] for p in traj.T]
-            path_y_list = [p[1] for p in traj.T]
+        path_x_list, path_y_list =traj_to_xy_list(traj)
 
         line = self.ax.plot(path_x_list, path_y_list, traj_type, label=label, **kwargs)
 
@@ -330,17 +338,7 @@ class EnvPlot:
         if points is None:
             return
 
-        if isinstance(points, list):
-            x_coordinates = [point[0] for point in points]
-            y_coordinates = [point[1] for point in points]
-
-        elif isinstance(points, np.ndarray):
-            if points.shape[1] > 1:
-                x_coordinates = [point[0] for point in points.T]
-                y_coordinates = [point[1] for point in points.T]
-            else:
-                x_coordinates = points[0]
-                y_coordinates = points[1]
+        x_coordinates, y_coordinates = points_to_xy_list(points)
 
         points_plot = self.ax.scatter(x_coordinates, y_coordinates, s, c, **kwargs)
 
@@ -416,6 +414,7 @@ class EnvPlot:
             self.dyna_line_list.append(box_line)
 
     def update_title(self) -> None:
+        """Update the figure title with current time/status or a custom title."""
         if self.title is not None:
             self.ax.set_title(self.title, pad=3)
         else:
@@ -450,7 +449,7 @@ class EnvPlot:
         self.saved_figure_kwargs.update(kwargs)
 
         if include_index or save_gif:
-            order = str(world_param.count).zfill(3)
+            order = str(world_param.count).zfill(4)
             full_name = fp + "/" + file_name + "_" + order + "." + file_format
         else:
             full_name = fp + "/" + file_name + "." + file_format
@@ -522,6 +521,14 @@ class EnvPlot:
     def logger(self):
         return env_param.logger
 
+    @property
+    def x_range(self):
+        return self.world.x_range
+
+    @property
+    def y_range(self):
+        return self.world.y_range
+
 
 def linewidth_from_data_units(
     linewidth: float, axis: Any, reference: str = "y"
@@ -556,3 +563,288 @@ def linewidth_from_data_units(
     length *= 72
     # Scale linewidth to value range
     return linewidth * (length / value_range)
+
+
+def draw_patch(
+    ax: Any,
+    shape: str,
+    state: Optional[np.ndarray] = None,
+    radius: Optional[float] = None,
+    vertices: Optional[np.ndarray] = None,
+    color: Optional[str] = None,
+    zorder: Optional[int] = None,
+    linestyle: Optional[str] = None,
+    **kwargs: Any,
+) -> Any:
+    """
+    Draw a geometric element (patch or line) on the given axes.
+
+    Supported shapes and expected inputs (refer to irsim.world.object_base plotting patterns):
+    - circle: use ``state`` (x,y,theta) and ``radius``; created at origin and transformed
+    - rectangle: prefer ``vertices`` (2xN) else use ``width``/``height`` with ``state`` transform
+    - polygon: use ``vertices`` (2xN)
+    - ellipse: use ``width``/``height`` with ``state`` transform
+    - wedge: use ``radius`` and either ``theta1``/``theta2`` (deg) or ``fov`` (rad); transformed by ``state``
+    - arrow: use ``state`` for position/orientation or provide ``theta``; supports ``arrow_length`` and ``arrow_width``
+    - line|linestring: use ``vertices`` (2xN) to draw a line element
+
+    Styling:
+    - color/edgecolor/facecolor, alpha, zorder, linestyle are respected when applicable.
+
+    Returns the created matplotlib artist (patch, line, or list from ax.plot).
+    """
+
+    created_element = None
+
+    # Normalize some common styling kwargs
+    facecolor = kwargs.pop("facecolor", None)
+    edgecolor = kwargs.pop("edgecolor", None)
+    alpha = kwargs.pop("alpha", None)
+    fill = kwargs.pop("fill", None)
+
+    state = state if state is not None else np.zeros((3, 1))
+
+    # Circle
+    if shape == "circle":
+        if radius is None:
+            raise ValueError("circle requires radius")
+
+        use_radius = radius
+        # Create at origin; translate/rotate with transform
+        patch = Circle((0.0, 0.0), use_radius)
+        created_element = ax.add_patch(patch)
+        set_patch_property(
+            created_element,
+            ax,
+            state=state,
+            color=color,
+            facecolor=facecolor,
+            edgecolor=edgecolor,
+            alpha=alpha,
+            zorder=zorder,
+            linestyle=linestyle,
+            fill=fill,
+        )
+
+    # Rectangle (prefer vertices; otherwise width/height + transform)
+    elif shape == "rectangle":
+        if vertices is not None:
+            patch = Polygon(vertices.T)
+            created_element = ax.add_patch(patch)
+            set_patch_property(
+                created_element,
+                ax,
+                state=state,  # vertices are absolute
+                color=color,
+                facecolor=facecolor,
+                edgecolor=edgecolor,
+                alpha=alpha,
+                zorder=zorder,
+                linestyle=linestyle,
+                fill=fill,
+            )
+        else:
+            width = kwargs.pop("width", None)
+            height = kwargs.pop("height", None)
+            if width is None or height is None:
+                raise ValueError("rectangle requires either vertices or width/height")
+
+            xy = (float(vertices[0, 0]), float(vertices[1, 0]))
+            patch = Rectangle(xy, width, height)
+            created_element = ax.add_patch(patch)
+            set_patch_property(
+                created_element,
+                ax,
+                state=state,
+                color=color,
+                facecolor=facecolor,
+                edgecolor=edgecolor,
+                alpha=alpha,
+                zorder=zorder,
+                linestyle=linestyle,
+                fill=fill,
+            )
+
+    # Polygon
+    elif shape == "polygon":
+        if vertices is None:
+            raise ValueError("polygon requires vertices (2xN)")
+        patch = Polygon(vertices.T)
+        created_element = ax.add_patch(patch)
+        set_patch_property(
+            created_element,
+            ax,
+            state=state,  # vertices are absolute
+            color=color,
+            facecolor=facecolor,
+            edgecolor=edgecolor,
+            alpha=alpha,
+            zorder=zorder,
+            linestyle=linestyle,
+            fill=fill,
+        )
+
+    # Ellipse
+    elif shape == "ellipse":
+        width = kwargs.pop("width", None)
+        height = kwargs.pop("height", None)
+        if width is None or height is None:
+            raise ValueError("ellipse requires width and height")
+        patch = Ellipse((0.0, 0.0), width, height, angle=0.0)
+        created_element = ax.add_patch(patch)
+        set_patch_property(
+            created_element,
+            ax,
+            state=state,
+            color=color,
+            facecolor=facecolor,
+            edgecolor=edgecolor,
+            alpha=alpha,
+            zorder=zorder,
+            linestyle=linestyle,
+            fill=fill,
+        )
+
+    # Wedge (FOV)
+    elif shape == "wedge":
+        use_radius = radius if radius is not None else kwargs.pop("radius", None)
+        if use_radius is None:
+            raise ValueError("wedge requires radius")
+        if "theta1" in kwargs and "theta2" in kwargs:
+            theta1 = kwargs.pop("theta1")
+            theta2 = kwargs.pop("theta2")
+        else:
+            # build from fov in radians centered at 0
+            fov = kwargs.pop("fov", np.pi)
+            theta1 = -180 * fov / (2 * np.pi)
+            theta2 = 180 * fov / (2 * np.pi)
+        patch = Wedge((0.0, 0.0), use_radius, theta1, theta2)
+        created_element = ax.add_patch(patch)
+        set_patch_property(
+            created_element,
+            ax,
+            state=state,
+            color=color,
+            facecolor=facecolor,
+            edgecolor=edgecolor,
+            alpha=alpha,
+            zorder=zorder,
+            linestyle=linestyle,
+            fill=fill,
+        )
+
+    # Arrow (velocity/heading)
+    elif shape == "arrow":
+        arrow_length = kwargs.pop("arrow_length", 0.4)
+        arrow_width = kwargs.pop("arrow_width", 0.6)
+        # Orientation: use provided theta or state[2]
+        theta = kwargs.pop("theta", float(state[2, 0]) if state.shape[0] >= 3 else 0.0)
+        x = float(state[0, 0])
+        y = float(state[1, 0])
+        dx = float(arrow_length * cos(theta))
+        dy = float(arrow_length * sin(theta))
+        patch = Arrow(x, y, dx, dy, width=arrow_width)
+        created_element = ax.add_patch(patch)
+        set_patch_property(
+            created_element,
+            ax,
+            state=None,  # absolute coords already applied
+            color=color if color is not None else kwargs.pop("arrow_color", None),
+            alpha=alpha,
+            zorder=zorder if zorder is not None else kwargs.pop("arrow_zorder", None),
+            fill=fill,
+        )
+
+    # Line / LineString
+    elif shape in ("line", "linestring"):
+        if vertices is None:
+            raise ValueError("line/linestring requires vertices (2xN)")
+        if isinstance(ax, Axes3D):
+            line3d = art3d.Line3D(
+                vertices[0, :], vertices[1, :], zs=kwargs.pop("z", np.zeros((3,)))
+            )
+            if color is not None:
+                line3d.set_color(color)
+            if alpha is not None:
+                line3d.set_alpha(alpha)
+            if zorder is not None:
+                line3d.set_zorder(zorder)
+            if fill is not None:
+                line3d.set_fill(fill)
+            ax.add_line(line3d)
+            created_element = line3d
+        else:
+            line2d = Line2D(vertices[0, :], vertices[1, :])
+            if linestyle is not None:
+                line2d.set_linestyle(linestyle)
+            if color is not None:
+                line2d.set_color(color)
+            if alpha is not None:
+                line2d.set_alpha(alpha)
+            if zorder is not None:
+                line2d.set_zorder(zorder)
+            ax.add_line(line2d)
+            created_element = line2d
+
+    else:
+        raise ValueError(f"Unsupported shape type: {shape}")
+
+    # 3D conversion for patches if needed
+    if (
+        isinstance(ax, Axes3D)
+        and created_element is not None
+        and shape not in ("line", "linestring")
+    ):
+        art3d.patch_2d_to_3d(created_element, z=kwargs.pop("z", 0), zdir="z")
+
+    return created_element
+
+
+def set_patch_property(
+    element: Any,
+    ax: Any,
+    state: Optional[np.ndarray] = None,
+    **kwargs: Any,
+) -> None:
+    """
+    Apply transform and style properties to a patch/artist, matching ObjectBase semantics.
+
+    - If ``state`` provided with at least 3 rows, apply rotation by theta and translation by (x, y).
+    - Apply color/facecolor/edgecolor, alpha, zorder, linestyle when supported.
+    """
+
+    # Transform by state (x, y, theta)
+    if state is not None and state.shape[0] >= 3:
+        x = float(state[0, 0])
+        y = float(state[1, 0])
+        theta = float(state[2, 0])
+        transform = mtransforms.Affine2D().rotate(theta).translate(x, y) + ax.transData
+        if hasattr(element, "set_transform"):
+            element.set_transform(transform)
+
+    # Styling
+    color = kwargs.get("color")
+    facecolor = kwargs.get("facecolor")
+    edgecolor = kwargs.get("edgecolor")
+    alpha = kwargs.get("alpha")
+    zorder = kwargs.get("zorder")
+    linestyle = kwargs.get("linestyle")
+    linewidth = kwargs.get("linewidth")
+    fill = kwargs.get("fill")
+
+    if color is not None and hasattr(element, "set_color"):
+        element.set_color(color)
+    if facecolor is not None and hasattr(element, "set_facecolor"):
+        element.set_facecolor(facecolor)
+    if edgecolor is not None and hasattr(element, "set_edgecolor"):
+        element.set_edgecolor(edgecolor)
+    if alpha is not None and hasattr(element, "set_alpha"):
+        element.set_alpha(alpha)
+    if zorder is not None and hasattr(element, "set_zorder"):
+        element.set_zorder(zorder)
+    if linestyle is not None and hasattr(element, "set_linestyle"):
+        element.set_linestyle(linestyle)
+    if linewidth is not None and hasattr(element, "set_linewidth"):
+        element.set_linewidth(linewidth)
+    if fill is not None and hasattr(element, "set_fill"):
+        element.set_fill(fill)
