@@ -10,6 +10,7 @@ import numpy as np
 from shapely.affinity import affine_transform
 
 from irsim.config import env_param
+from irsim.util.random import rng
 
 
 def file_check(
@@ -431,64 +432,6 @@ def diff_to_omni(state_ori: float, vel_diff: np.ndarray) -> np.ndarray:
     return np.array([[vx], [vy]])
 
 
-def time_it(name: str = "Function") -> Any:
-    """
-    Decorator to measure function execution time.
-
-    Args:
-        name (str): Function name for logging (default "Function").
-        print (bool): Whether to print execution time (default True).
-
-    Returns:
-        function: Wrapped function with timing.
-    """
-
-    def decorator(func):
-        def wrapper(*args, **kwargs):
-            wrapper.count += 1
-            start = time.time()
-            result = func(*args, **kwargs)
-            end = time.time()
-            wrapper.func_count += 1
-            print(f"{name} execute time {(end - start):.6f} seconds")
-            return result
-
-        wrapper.count = 0
-        wrapper.func_count = 0
-        return wrapper
-
-    return decorator
-
-
-def time_it2(name: str = "Function") -> Any:
-    """
-    Decorator to measure function execution time with instance attribute check.
-
-    Args:
-        name (str): Function name for logging (default "Function").
-
-    Returns:
-        function: Wrapped function with timing.
-    """
-
-    def decorator(func):
-        def wrapper(self, *args, **kwargs):
-            wrapper.count += 1
-            start = time.time()
-            result = func(self, *args, **kwargs)
-            end = time.time()
-            wrapper.func_count += 1
-            if self.time_print:
-                print(f"{name} execute time {(end - start):.6f} seconds")
-            return result
-
-        wrapper.count = 0
-        wrapper.func_count = 0
-        return wrapper
-
-    return decorator
-
-
 def cross_product(o: list[float], a: list[float], b: list[float]) -> float:
     """
     Compute the cross product of vectors OA and OB.
@@ -626,7 +569,7 @@ def random_point_range(
     if isinstance(range_high, list):
         range_high = np.c_[range_high]
 
-    return np.random.uniform(range_low, range_high)
+    return rng.uniform(range_low, range_high)
 
 
 def is_2d_list(data: Union[list, deque]) -> bool:
@@ -646,3 +589,207 @@ def is_2d_list(data: Union[list, deque]) -> bool:
             return True
 
     return False
+
+
+# decorator
+
+
+def time_it(name: str = "Function") -> Any:
+    """
+    Decorator to measure function execution time.
+
+    Args:
+        name (str): Function name for logging (default "Function").
+        print (bool): Whether to print execution time (default True).
+
+    Returns:
+        function: Wrapped function with timing.
+    """
+
+    def decorator(func):
+        def wrapper(*args, **kwargs):
+            wrapper.count += 1
+            start = time.time()
+            result = func(*args, **kwargs)
+            end = time.time()
+            wrapper.func_count += 1
+            print(f"{name} execute time {(end - start):.6f} seconds")
+            return result
+
+        wrapper.count = 0
+        wrapper.func_count = 0
+        return wrapper
+
+    return decorator
+
+
+def normalize_actions(func):
+    """
+    Decorator to normalize (action, action_id) into an aligned actions list.
+
+    The wrapped method must belong to a class that has a ``objects`` attribute.
+    It will receive an extra keyword argument ``aligned_actions`` (length == len(self.objects)).
+    """
+
+    def wrapper(self, action=None, action_id=0, *args, **kwargs):
+        num_objects = len(getattr(self, "objects", []))
+        actions = [None] * num_objects
+
+        if action is not None:
+            if isinstance(action, list):
+                if isinstance(action_id, list):
+                    for a, ai in zip(action, action_id):
+                        actions[int(ai)] = a
+                else:
+                    start = int(action_id)
+                    actions[start : start + len(action)] = action[:]
+            elif isinstance(action, np.ndarray):
+                if isinstance(action_id, list):
+                    for ai in action_id:
+                        actions[int(ai)] = action
+                else:
+                    actions[int(action_id)] = action
+
+        # Call the original function with normalized actions and a neutral action_id
+        return func(self, actions, 0, *args, **kwargs)
+
+    return wrapper
+
+
+def time_it2(name: str = "Function") -> Any:
+    """
+    Decorator to measure function execution time with instance attribute check.
+
+    Args:
+        name (str): Function name for logging (default "Function").
+
+    Returns:
+        function: Wrapped function with timing.
+    """
+
+    def decorator(func):
+        def wrapper(self, *args, **kwargs):
+            wrapper.count += 1
+            start = time.time()
+            result = func(self, *args, **kwargs)
+            end = time.time()
+            wrapper.func_count += 1
+            if self.time_print:
+                print(f"{name} execute time {(end - start):.6f} seconds")
+            return result
+
+        wrapper.count = 0
+        wrapper.func_count = 0
+        return wrapper
+
+    return decorator
+
+
+def to_numpy(
+    data: Any,
+    default: Optional[np.ndarray] = None,
+    expected_shape: Optional[tuple[int, ...]] = None,
+) -> Optional[np.ndarray]:
+    """
+    Convert input to numpy array and optionally reshape.
+
+    - If data is None: return default (reshaped if expected_shape provided).
+    - If data is a list: convert to ndarray. 1D lists become column vectors.
+    - If data is a 1D ndarray: convert to column vector.
+    - If expected_shape is provided: reshape the resulting array to it.
+    """
+
+    if data is None:
+        if default is None:
+            return None
+        return default if expected_shape is None else default.reshape(expected_shape)
+
+    arr = np.array(data) if isinstance(data, list) else data
+
+    if isinstance(arr, np.ndarray) and arr.ndim == 1:
+        arr = arr[:, np.newaxis]
+
+    if expected_shape is not None:
+        arr = np.asarray(arr).reshape(expected_shape)
+
+    return arr
+
+
+def traj_to_xy_list(
+    traj: Any, three_d: bool = False
+) -> tuple[list[float], list[float]]:
+    """
+    Convert trajectory to a list of [x, y].
+
+    Args:
+        traj (list or np.ndarray): list of points or array of points [x, y, theta].
+
+    Returns:
+        tuple: A tuple of lists containing x and y coordinates of the trajectory
+            x_list (list): List of x coordinates.
+            y_list (list): List of y coordinates.
+    """
+    if isinstance(traj, list):
+        x_list = [p[0, 0] for p in traj]
+        y_list = [p[1, 0] for p in traj]
+        if three_d:
+            z_list = [p[2, 0] for p in traj]
+    elif isinstance(traj, np.ndarray):
+        if traj.shape[1] > 1:
+            x_list = [p[0] for p in traj.T]
+            y_list = [p[1] for p in traj.T]
+            if three_d:
+                z_list = [p[2] for p in traj.T]
+        else:
+            x_list = [traj[0]]
+            y_list = [traj[1]]
+            if three_d:
+                z_list = [traj[2]]
+    else:
+        raise ValueError(f"Invalid trajectory type: {type(traj)}")
+
+    if three_d:
+        return x_list, y_list, z_list
+    return x_list, y_list
+
+
+def points_to_xy_list(
+    points: Any, three_d: bool = False
+) -> tuple[list[float], list[float]]:
+    """
+    Convert points to a list of [x, y].
+
+    Args:
+        points (list or np.ndarray): list of points or array of points [x, y].
+        three_d (bool): Whether the points are 3D.
+    Returns:
+        tuple: A tuple of lists containing x and y coordinates of the points
+            x_list (list): List of x coordinates.
+            y_list (list): List of y coordinates.
+    """
+
+    if points is None:
+        return [], []
+
+    if isinstance(points, list):
+        x_list = [point[0] for point in points]
+        y_list = [point[1] for point in points]
+        if three_d:
+            z_list = [point[2] for point in points]
+    elif isinstance(points, np.ndarray):
+        if points.shape[1] > 1:
+            x_list = [point[0] for point in points.T]
+            y_list = [point[1] for point in points.T]
+            if three_d:
+                z_list = [point[2] for point in points.T]
+        else:
+            x_list = [points[0]]
+            y_list = [points[1]]
+            if three_d:
+                z_list = [points[2]]
+    else:
+        raise ValueError(f"Invalid points type: {type(points)}")
+
+    if three_d:
+        return x_list, y_list, z_list
+    return x_list, y_list
