@@ -5,15 +5,11 @@ from dataclasses import dataclass
 from math import atan2, cos, inf, pi, sin
 from typing import Any, Optional, Union
 
-import matplotlib as mpl
 import matplotlib.transforms as mtransforms
-import mpl_toolkits.mplot3d.art3d as art3d
 import numpy as np
 import shapely
 from matplotlib import image
-from matplotlib.lines import Line2D
-from matplotlib.patches import Arrow, Circle, Rectangle, Wedge
-from matplotlib.patches import Polygon as MplPolygon
+from matplotlib.patches import Arrow, Circle, Wedge
 from mpl_toolkits.mplot3d import Axes3D
 from shapely.geometry import MultiLineString
 from shapely.geometry.base import BaseGeometry
@@ -21,7 +17,7 @@ from shapely.strtree import STRtree
 
 from irsim.config import env_param, world_param
 from irsim.config.path_param import path_manager
-from irsim.env.env_plot import linewidth_from_data_units
+from irsim.env.env_plot import draw_patch, linewidth_from_data_units, set_patch_property
 from irsim.lib import Behavior, GeometryFactory, KinematicsFactory
 from irsim.util.util import (
     WrapTo2Pi,
@@ -32,6 +28,7 @@ from irsim.util.util import (
     is_2d_list,
     random_point_range,
     relative_position,
+    to_numpy,
     vertices_transform,
 )
 from irsim.world.sensors.sensor_factory import SensorFactory
@@ -56,6 +53,7 @@ class ObjectInfo:
     h: np.ndarray
     cone_type: str
     convex_flag: bool
+    name: str
 
     def add_property(self, key, value):
         setattr(self, key, value)
@@ -201,6 +199,7 @@ class ObjectBase:
         unobstructed: bool = False,
         fov: Optional[float] = None,
         fov_radius: Optional[float] = None,
+        name: Optional[str] = None,
         **kwargs,
     ) -> None:
         """
@@ -276,6 +275,8 @@ class ObjectBase:
         self._velocity = np.c_[velocity]
         self._init_velocity = np.c_[velocity]
 
+        self._name = name
+
         # Set goal points
         self._goal = (
             deque(goal)
@@ -327,6 +328,7 @@ class ObjectBase:
             self.h,
             self.cone_type,
             self.convex_flag,
+            self.name,
         )
 
         self.obstacle_info = None
@@ -397,7 +399,12 @@ class ObjectBase:
         """reset the id iterator"""
         cls.id_iter = itertools.count(start, step)
 
-    def step(self, velocity: Optional[np.ndarray] = None, **kwargs: Any):
+    def step(
+        self,
+        velocity: Optional[np.ndarray] = None,
+        sensor_step: bool = True,
+        **kwargs: Any,
+    ):
         """
         Perform a single simulation step, updating the object's state and sensors.
 
@@ -437,7 +444,10 @@ class ObjectBase:
         self._state = next_state
         self._velocity = behavior_vel
         self._geometry = self.gf.step(self.state)
-        self.sensor_step()
+
+        if sensor_step:
+            self.sensor_step()
+
         self.post_process()
         self.trajectory.append(self.state.copy())
         return next_state
@@ -462,7 +472,7 @@ class ObjectBase:
             self.stop_flag = any(not obj.unobstructed for obj in self.collision_obj)
 
         elif world_param.collision_mode == "reactive":
-            "to be implemented"
+            "currently same as unobstructed: to be further implemented"
             pass
 
         elif world_param.collision_mode == "unobstructed":
@@ -502,6 +512,7 @@ class ObjectBase:
         if diff < self.goal_threshold:
             if len(self._goal) == 1:
                 self.arrive_flag = True
+
             else:
                 self._goal.popleft()
                 self.arrive_flag = False
@@ -593,12 +604,7 @@ class ObjectBase:
             )
 
         else:
-            if isinstance(velocity, list):
-                velocity = np.c_[velocity]
-            if velocity.ndim == 1:
-                velocity = velocity[:, np.newaxis]
-
-            assert velocity.shape == self.vel_shape
+            velocity = to_numpy(velocity, expected_shape=self.vel_shape)
 
             behavior_vel = velocity
 
@@ -772,17 +778,8 @@ class ObjectBase:
         """
         if state is None:
             state = [0, 0, 0]
-        if isinstance(state, list):
-            assert len(state) == self.state_dim, (
-                f"The state dimension is not correct. Your input state length is {len(state)} and the state dimension should be {self.state_dim}"
-            )
-            temp_state = np.c_[state]
 
-        elif isinstance(state, np.ndarray):
-            assert state.shape[0] == self.state_dim, (
-                f"The state dimension is not correct. Your input state dimension is {state.shape[0]} and the state dimension should be {self.state_dim}"
-            )
-            temp_state = state.reshape(self.state_dim, 1)
+        temp_state = to_numpy(state, expected_shape=self.state_shape)
 
         if init:
             self._init_state = temp_state.copy()
@@ -801,19 +798,9 @@ class ObjectBase:
             init (bool): Whether to set the initial velocity (default False).
         """
 
-        if velocity is None:
-            velocity = [0, 0]
-        if isinstance(velocity, list):
-            assert len(velocity) == self.vel_dim, (
-                f"The velocity dimension is not correct. Your input velocity length is {len(velocity)} and the velocity dimension should be {self.vel_dim}"
-            )
-            temp_velocity = np.c_[velocity]
-
-        elif isinstance(velocity, np.ndarray):
-            assert velocity.shape[0] == self.vel_dim, (
-                f"The velocity dimension is not correct. Your input velocity dimension is {velocity.shape[0]} and the velocity dimension should be {self.vel_dim}"
-            )
-            temp_velocity = velocity.reshape(self.vel_dim, 1)
+        temp_velocity = to_numpy(
+            velocity, default=np.zeros(self.vel_shape), expected_shape=self.vel_shape
+        )
 
         if init:
             self._init_velocity = temp_velocity.copy()
@@ -823,6 +810,10 @@ class ObjectBase:
     def set_original_geometry(self, geometry: BaseGeometry):
         """
         Set the original geometry of the object.
+
+        Args:
+            geometry (BaseGeometry): Shapely geometry to use as the new original geometry.
+                Subsequent geometry updates will be transformed from this base.
         """
         self.gf._original_geometry = geometry
 
@@ -1031,7 +1022,12 @@ class ObjectBase:
         self._plot(ax, state, vertices, **kwargs)
 
     def _init_plot(self, ax, **kwargs):
-        """Initialize plotting elements using zero state and initial vertices."""
+        """
+        Initialize plotting elements using zero state and initial vertices.
+
+        Returns:
+            list: Names of plot attributes created (e.g., 'object_patch', 'goal_patch').
+        """
         return self._plot(
             ax, self.original_state, self.original_vertices, initial=True, **kwargs
         )
@@ -1049,7 +1045,7 @@ class ObjectBase:
 
             Object visualization properties:
                 - obj_linestyle (str): Line style for object outline (e.g., '-', '--', ':', '-.').
-                - obj_zorder (int): Z-order (drawing layer) for object elements.
+                - obj_zorder (int): Z-order (drawing layer) for object elements; defaults to 3 for robots, 1 for obstacles.
                 - obj_color (str): Color of the object.
                 - obj_alpha (float): Transparency of the object (0.0 to 1.0).
 
@@ -1160,8 +1156,9 @@ class ObjectBase:
 
     def _step_plot(self, **kwargs):
         """
-        Update the positions and properties of all plot elements created in _init_plot for 2D plotting. In the 3D case, the patches are updated using draw_component and clear_component method.
-        Elements are updated using transforms for patches and set_data for lines, based on the object's current state.
+        Update the positions and properties of all plot elements created in _init_plot.
+        Elements are updated using transforms for patches and set_data/set_data_3d for lines,
+        based on the object's current state, in both 2D and 3D.
 
         Update methods by element type:
         - Patches (object_patch, goal_patch, arrow_patch, fov_patch): Updated using matplotlib transforms
@@ -1232,18 +1229,13 @@ class ObjectBase:
                 if attr == "object_patch":
                     # For patches created at origin in _init_plot, use transform to position them
                     obj_kwargs = {
-                        "color": kwargs.get("obj_color"),
+                        "color": kwargs.get("obj_color", self.color),
                         "alpha": kwargs.get("obj_alpha"),
                         "zorder": kwargs.get("obj_zorder"),
+                        "linestyle": kwargs.get("obj_linestyle"),
                     }
-                    # Filter out None values
-                    obj_kwargs = {k: v for k, v in obj_kwargs.items() if v is not None}
 
-                    self.set_element_property(element, self.state, **obj_kwargs)
-
-                    # Update object patch properties that are not handled by set_element_property
-                    if "obj_linestyle" in kwargs:
-                        element.set_linestyle(kwargs["obj_linestyle"])
+                    set_patch_property(element, self.ax, state=self.state, **obj_kwargs)
 
                 elif attr == "object_line":
                     # For lines, use set_data to update coordinates (works for both 2D and 3D)
@@ -1305,10 +1297,6 @@ class ObjectBase:
                         "zorder": kwargs.get("goal_zorder"),
                     }
 
-                    goal_kwargs = {
-                        k: v for k, v in goal_kwargs.items() if v is not None
-                    }
-
                     if self.goal is None:
                         element.set_visible(False)
                         continue
@@ -1322,7 +1310,9 @@ class ObjectBase:
                             self.goal, (0, 1), "constant", constant_values=0
                         )
 
-                    self.set_element_property(element, goal_state, **goal_kwargs)
+                    set_patch_property(
+                        element, self.ax, state=goal_state, **goal_kwargs
+                    )
 
                 elif attr == "arrow_patch":
                     # Update arrow patch using set_element_property
@@ -1341,12 +1331,10 @@ class ObjectBase:
                             "alpha": kwargs.get("arrow_alpha"),
                             "zorder": kwargs.get("arrow_zorder"),
                         }
-                        # Filter out None values
-                        arrow_kwargs = {
-                            k: v for k, v in arrow_kwargs.items() if v is not None
-                        }
 
-                        self.set_element_property(element, arrow_state, **arrow_kwargs)
+                        set_patch_property(
+                            element, self.ax, state=arrow_state, **arrow_kwargs
+                        )
 
                 elif attr == "trajectory_line":
                     # Update trajectory line using set_data (works for both 2D and 3D)
@@ -1396,24 +1384,20 @@ class ObjectBase:
                             "alpha": kwargs.get("fov_alpha"),
                             "zorder": kwargs.get("fov_zorder"),
                         }
-                        # Filter out None values
-                        fov_kwargs = {
-                            k: v for k, v in fov_kwargs.items() if v is not None
-                        }
 
-                        self.set_element_property(element, fov_state, **fov_kwargs)
-
-                        # Update special FOV properties that are not handled by set_element_property
-                        if "fov_color" in kwargs:
-                            element.set_facecolor(kwargs["fov_color"])
-
-                        if "fov_edge_color" in kwargs:
-                            element.set_edgecolor(kwargs["fov_edge_color"])
+                        set_patch_property(
+                            element, self.ax, state=fov_state, **fov_kwargs
+                        )
 
         # Update text position using set_position (works for both 2D and 3D)
         if hasattr(self, "abbr_text"):
             text = self.abbr_text
-            text_position = [-self.radius - 0.1, self.radius + 0.1]
+            # Prefer runtime kwargs, then initial plot kwargs, fallback to default
+            default_text_pos = [-self.radius - 0.1, self.radius + 0.1]
+            text_position = kwargs.get(
+                "text_position",
+                self.plot_kwargs.get("text_position", default_text_pos),
+            )
 
             text.set_position((x + text_position[0], y + text_position[1]))
 
@@ -1432,30 +1416,14 @@ class ObjectBase:
 
         # Handle trail plotting (creates new elements each time)
         if self.show_trail and world_param.count % self.trail_freq == 0:
-            self.plot_trail(self.ax, self.state, self.vertices, **self.plot_kwargs)
+            self.plot_trail(
+                self.ax, self.state, self.original_vertices, **self.plot_kwargs
+            )
 
         # Update sensors
         if self.show_sensor:
             [sensor.step_plot() for sensor in self.sensors]
 
-    def set_element_property(self, element, state, **kwargs):
-        x = state[0, 0]
-        y = state[1, 0]
-        theta = state[2, 0]
-
-        trans = (
-            mpl.transforms.Affine2D().rotate(theta).translate(x, y) + self.ax.transData
-        )
-
-        element.set_transform(trans)
-
-        if "color" in kwargs:
-            element.set_color(kwargs["color"])
-        if "alpha" in kwargs:
-            element.set_alpha(kwargs["alpha"])
-        if "zorder" in kwargs:
-            element.set_zorder(kwargs["zorder"])
-
     def plot_object(
         self,
         ax,
@@ -1482,10 +1450,10 @@ class ObjectBase:
             None
 
         Raises:
-            ValueError: When object shape is not supported
+            Exception: If the underlying patch creation fails (e.g., unsupported shape or backend issues).
         """
         obj_linestyle = kwargs.get("obj_linestyle", "-")
-        obj_zorder = kwargs.get("obj_zorder", 2) if self.role == "robot" else 1
+        obj_zorder = kwargs.get("obj_zorder", 3) if self.role == "robot" else 1
 
         state = self.state if state is None else state
         vertices = self.vertices if vertices is None else vertices
@@ -1493,60 +1461,19 @@ class ObjectBase:
         # Handle 3D plot or no description case
         if self.description is None or isinstance(ax, Axes3D):
             try:
-                if self.shape == "circle":
-                    object_patch = Circle(
-                        xy=(float(state[0, 0]), float(state[1, 0])),
+                if self.shape != "map":
+                    self.object_patch = draw_patch(
+                        ax,
+                        shape=self.shape,
+                        state=state,
                         radius=self.radius,
+                        vertices=vertices,
                         color=self.color,
                         linestyle=obj_linestyle,
-                    )
-                    object_patch.set_zorder(obj_zorder)
-
-                    if isinstance(ax, Axes3D):
-                        art3d.patch_2d_to_3d(object_patch, z=self.z, zdir="z")
-
-                    ax.add_patch(object_patch)
-                    self.plot_patch_list.append(object_patch)
-                    self.object_patch = object_patch
-
-                elif self.shape in ["polygon", "rectangle"]:
-                    object_patch = MplPolygon(
-                        xy=vertices.T, color=self.color, linestyle=obj_linestyle
+                        zorder=obj_zorder,
                     )
 
-                    object_patch.set_zorder(obj_zorder)
-
-                    if isinstance(ax, Axes3D):
-                        art3d.patch_2d_to_3d(object_patch, z=self.z, zdir="z")
-
-                    ax.add_patch(object_patch)
-                    self.plot_patch_list.append(object_patch)
-                    self.object_patch = object_patch
-
-                elif self.shape == "linestring":
-                    if isinstance(ax, Axes3D):
-                        object_line = art3d.Line3D(
-                            vertices[0, :],
-                            vertices[1, :],
-                            zs=self.z * np.ones((3,)),
-                            color=self.color,
-                        )
-                    else:
-                        object_line = Line2D(
-                            vertices[0, :],
-                            vertices[1, :],
-                            color=self.color,
-                            linestyle=obj_linestyle,
-                        )
-                    object_line.set_zorder(obj_zorder)
-                    ax.add_line(object_line)
-                    self.plot_patch_list.append(object_line)
-                    self.object_line = object_line
-
-                elif self.shape == "map":
-                    return
-                else:
-                    raise ValueError(f"Unsupported shape type: {self.shape}")
+                    self.plot_patch_list.append(self.object_patch)
 
             except Exception as e:
                 self.logger.error(f"Error occurred while plotting object: {e!s}")
@@ -1654,7 +1581,7 @@ class ObjectBase:
 
         solid_capstyle = "round" if self.shape == "circle" else "butt"
 
-        traj_line = ax.plot(
+        self.trajectory_line = ax.plot(
             x_list,
             y_list,
             color=traj_color,
@@ -1666,8 +1593,7 @@ class ObjectBase:
             zorder=traj_zorder,
         )
 
-        self.plot_line_list.append(traj_line)
-        self.trajectory_line = traj_line
+        self.plot_line_list.append(self.trajectory_line)
 
     def plot_goal(
         self,
@@ -1685,7 +1611,7 @@ class ObjectBase:
         Args:
             ax: Matplotlib axis.
             goal_state: State of the goal (x, y, r_phi) defining goal position and orientation.
-                       If None, uses [0, 0, 0]. Defaults to None.
+                       If None, nothing is plotted. Defaults to None.
             vertices: Vertices for polygon/rectangle goal shapes.
                      If None, uses original_vertices. Defaults to None.
             goal_color (str): Color of the goal marker. Defaults to be the color of the object.
@@ -1693,32 +1619,23 @@ class ObjectBase:
             goal_alpha (float): Transparency of the goal marker. Defaults to 0.5.
         """
 
-        if goal_color is None:
-            goal_color = self.color
+        goal_color = self.color if goal_color is None else goal_color
 
         if goal_state is None:
             return
 
-        if self.shape == "circle":
-            goal_patch = Circle(
-                xy=(float(goal_state[0, 0]), float(goal_state[1, 0])),
-                radius=self.radius,
-                color=goal_color,
-                alpha=goal_alpha,
-            )
-        else:
-            if vertices is None:
-                return
-            goal_patch = MplPolygon(xy=vertices.T, color=goal_color)
+        self.goal_patch = draw_patch(
+            ax,
+            shape=self.shape,
+            state=goal_state,
+            radius=self.radius,
+            vertices=vertices,
+            color=goal_color,
+            alpha=goal_alpha,
+            zorder=goal_zorder,
+        )
 
-        if isinstance(ax, Axes3D):
-            art3d.patch_2d_to_3d(goal_patch, z=self.z)
-
-        goal_patch.set_zorder(int(goal_zorder) if goal_zorder is not None else 1)
-        ax.add_patch(goal_patch)
-
-        self.plot_patch_list.append(goal_patch)
-        self.goal_patch = goal_patch
+        self.plot_patch_list.append(self.goal_patch)
 
     def plot_text(self, ax, state: Optional[np.ndarray] = None, **kwargs):
         """
@@ -1736,6 +1653,9 @@ class ObjectBase:
                   default is [-radius-0.1, radius+0.1].
                 - text_zorder (int): Zorder of the text. Defaults to 2.
                 - text_alpha (float): Transparency of the text. Defaults to 1.
+
+        Note:
+            Subsequent updates via _step_plot honor the configured text_position if provided.
         """
 
         if state is None:
@@ -1752,7 +1672,7 @@ class ObjectBase:
         x, y = state[0, 0], state[1, 0]
 
         if isinstance(ax, Axes3D):
-            text = ax.text(
+            self.abbr_text = ax.text(
                 x + text_position[0],
                 y + text_position[1],
                 self.z,
@@ -1763,7 +1683,7 @@ class ObjectBase:
                 alpha=text_alpha,
             )
         else:
-            text = ax.text(
+            self.abbr_text = ax.text(
                 x + text_position[0],
                 y + text_position[1],
                 self.abbr,
@@ -1773,8 +1693,7 @@ class ObjectBase:
                 alpha=text_alpha,
             )
 
-        self.plot_text_list.append(text)
-        self.abbr_text = text
+        self.plot_text_list.append(self.abbr_text)
 
     def plot_arrow(
         self,
@@ -1809,32 +1728,24 @@ class ObjectBase:
         if arrow_color is None:
             arrow_color = "gold"
 
-        x = state[0, 0]
-        y = state[1, 0]
-
         theta = (
             atan2(velocity[1, 0], velocity[0, 0])
             if self.kinematics == "omni" and velocity is not None
             else state[2, 0]
         )
 
-        arrow = Arrow(
-            float(x),
-            float(y),
-            float(arrow_length * cos(theta)),
-            float(arrow_length * sin(theta)),
-            width=arrow_width,
+        self.arrow_patch = draw_patch(
+            ax,
+            shape="arrow",
+            state=state,
             color=arrow_color,
+            zorder=arrow_zorder,
+            arrow_length=arrow_length,
+            arrow_width=arrow_width,
+            theta=theta,
         )
 
-        if isinstance(ax, Axes3D):
-            art3d.patch_2d_to_3d(arrow, z=self.z)
-
-        arrow.set_zorder(arrow_zorder)
-        ax.add_patch(arrow)
-
-        self.plot_patch_list.append(arrow)
-        self.arrow_patch = arrow
+        self.plot_patch_list.append(self.arrow_patch)
 
     def plot_trail(
         self,
@@ -1851,7 +1762,7 @@ class ObjectBase:
             ax: Matplotlib axis.
             state: State of the object (x, y, r_phi) to determine trail position and orientation.
                    If None, uses the object's current state. Defaults to None.
-            vertices: Vertices of the object for polygon and rectangle trail shapes.
+            vertices: Original vertices of the object for polygon and rectangle trail shapes.
                      If None, uses the object's current vertices. Defaults to None.
             keep_trail_length (int): Number of steps to keep from the recent trajectory of trail.
             **kwargs: Additional plotting options:
@@ -1864,10 +1775,8 @@ class ObjectBase:
                 trail_zorder (int): Z-order of the trail.
         """
 
-        if state is None:
-            state = self.state
         if vertices is None:
-            vertices = self.vertices
+            vertices = self.original_vertices
 
         trail_type = kwargs.get("trail_type", self.shape)
         trail_edgecolor = kwargs.get("trail_edgecolor", self.color)
@@ -1877,63 +1786,23 @@ class ObjectBase:
         trail_color = kwargs.get("trail_color", self.color)
         trail_zorder = kwargs.get("trail_zorder", 0)
 
-        r_phi_ang = 180 * state[2, 0] / pi
+        # angle in degrees, no longer needed due to generic draw_patch usage
 
-        if trail_type == "rectangle":
-            start_x = vertices[0, 0]
-            start_y = vertices[1, 0]
-
-            trail = Rectangle(
-                xy=(float(start_x), float(start_y)),
-                width=self.length,
-                height=self.width,
-                angle=r_phi_ang,
-                edgecolor=trail_edgecolor,
-                fill=False,
-                alpha=trail_alpha,
-                linewidth=trail_linewidth,
-                facecolor=trail_color,
-                zorder=trail_zorder,
-            )
-
-            if isinstance(ax, Axes3D):
-                art3d.patch_2d_to_3d(trail, z=self.z)
-
-            ax.add_patch(trail)
-
-        elif trail_type == "polygon":
-            trail = MplPolygon(
-                vertices.T,
-                edgecolor=trail_color,
-                alpha=trail_alpha,
-                linewidth=trail_linewidth,
-                fill=False,
-                facecolor=trail_color,
-            )
-
-            if isinstance(ax, Axes3D):
-                art3d.patch_2d_to_3d(trail, z=self.z)
-
-            ax.add_patch(trail)
-
-        elif trail_type == "circle":
-            # For circle, use the state position as center
-            trail = Circle(
-                xy=(float(state[0, 0]), float(state[1, 0])),
-                radius=self.radius,
-                edgecolor=trail_edgecolor,
-                fill=trail_fill,
-                alpha=trail_alpha,
-                facecolor=trail_color,
-            )
-
-            if isinstance(ax, Axes3D):
-                art3d.patch_2d_to_3d(trail, z=self.z)
-
-            ax.add_patch(trail)
-
-        else:
-            raise ValueError(f"Invalid trail type: {trail_type}")
+        trail = draw_patch(
+            ax,
+            shape=trail_type,
+            state=state,
+            vertices=vertices,
+            radius=self.radius,
+            width=self.length,
+            height=self.width,
+            edgecolor=trail_edgecolor,
+            facecolor=trail_color,
+            fill=trail_fill,
+            alpha=trail_alpha,
+            linewidth=trail_linewidth,
+            zorder=trail_zorder,
+        )
 
         self.plot_trail_list.append(trail)
 
@@ -1954,45 +1823,41 @@ class ObjectBase:
                 fov_edge_color (str): Edge color of the field of view. Default is 'blue'.
                 fov_zorder (int): Z-order of the field of view. Default is 1.
                 fov_alpha (float): Transparency of the field of view. Default is 0.5.
+
+        Note:
+            No-op when FOV is not configured (fov or fov_radius is None).
         """
 
+        if self.fov is None or self.fov_radius is None:
+            return
+
         fov_color = kwargs.get("fov_color", "lightblue")
         fov_edge_color = kwargs.get("fov_edge_color", "blue")
-        fov_zorder = kwargs.get("fov_zorder", 0)
+        fov_zorder = kwargs.get("fov_zorder", 1)
         fov_alpha = kwargs.get("fov_alpha", 0.5)
 
         # Create FOV wedge at origin with no rotation (-fov/2 to +fov/2 around 0 degrees)
         start_degree = -180 * self.fov / (2 * pi)
         end_degree = 180 * self.fov / (2 * pi)
 
-        if abs(self.fov - 2 * pi) < 0.01:
-            fov_wedge = Circle(
-                (0, 0),
-                self.fov_radius,
-                facecolor=fov_color,
-                alpha=fov_alpha,
-                edgecolor=fov_edge_color,
-                zorder=fov_zorder,
-            )
-        else:
-            fov_wedge = Wedge(
-                (0, 0),  # Create at origin
-                self.fov_radius,
-                start_degree,
-                end_degree,
-                facecolor=fov_color,
-                alpha=fov_alpha,
-                edgecolor=fov_edge_color,
-                zorder=fov_zorder,
-            )
+        state = np.array([[0.0], [0.0], [0.0]])
 
-        if isinstance(ax, Axes3D):
-            art3d.patch_2d_to_3d(fov_wedge, z=self.z)
+        shape = "circle" if abs(self.fov - 2 * pi) < 0.01 else "wedge"
 
-        ax.add_patch(fov_wedge)
+        self.fov_patch = draw_patch(
+            ax,
+            shape=shape,
+            state=state,
+            radius=self.fov_radius,
+            theta1=start_degree,
+            theta2=end_degree,
+            facecolor=fov_color,
+            edgecolor=fov_edge_color,
+            alpha=fov_alpha,
+            zorder=fov_zorder,
+        )
 
-        self.plot_patch_list.append(fov_wedge)
-        self.fov_patch = fov_wedge
+        self.plot_patch_list.append(self.fov_patch)
 
     def plot_uncertainty(self, ax, **kwargs):
         """
@@ -2112,6 +1977,36 @@ class ObjectBase:
             center=self.position, radius=self.radius, vertices=self.vertices
         )
 
+    def get_desired_omni_vel(self, goal_threshold=0.1, normalized=False) -> np.ndarray:
+        """
+        Get the desired omnidirectional velocity of the object.
+
+        Args:
+            goal_threshold (float): Threshold for goal proximity.
+            normalized (bool): Whether to normalize the velocity.
+        """
+
+        if self.goal is None:
+            return np.zeros((2, 1))
+
+        dis, radian = relative_position(self.state, self.goal)
+
+        if dis > goal_threshold:
+            vx = self.vel_max[0, 0] * cos(radian)
+            vy = self.vel_max[1, 0] * sin(radian)
+        else:
+            vx = 0
+            vy = 0
+
+        if normalized:
+            length = np.linalg.norm([vx, vy])
+
+            if length > 1:
+                vx = vx / length
+                vy = vy / length
+
+        return np.array([[vx], [vy]])
+
     @property
     def name(self) -> str:
         """
@@ -2121,7 +2016,7 @@ class ObjectBase:
             str: The name of the object.
         """
 
-        return self.role + "_" + str(self.id)
+        return self._name if self._name is not None else self.role + "_" + str(self.id)
 
     @property
     def abbr(self) -> str:
